program fontConvert;

const
	black=0;
	white=255;

type
	tPcxPalRGB=packed record
		red,green,blue:byte;
	end;
  tPcxPal256=packed array[0..255] of tPcxPalRGB;
  tPcxArrayPal=packed array [0..15, 0..2] of byte;
  tPcxHeader=packed record
    manufacturer,version,encoding,bits_per_pixel:byte;
    xMin,yMin,xMax,yMax,hRes,vRes:word;
    palette:array[0..47] of byte;
    reserved:byte;
    color_planes:byte;
    bytes_per_line:word;
    palette_type:word;
    filler:array[0..57] of byte;
  end;
  
	tglKernedChar=array[0..5] of shortInt;
	tglKernedTable=array[32..127] of tglKernedChar;
	
	tglkFontHeader=record
		formatRevision,
		bufferWidth,bufferHeight,
		charWidth,charHeight:dword;
		kerningTable:tglKernedTable;
		reserved:array[0..127] of byte;
	end;
	
	tglKerningInfoLine=record
		command:longint;
		data:tglKernedChar;
	end;
	
  
function delimString(st:string; delim:char; var t:word):string;
var
  tst:string;
begin
  tst:='';
  while not(st[t]=delim) and (t<=length(st)) do begin
    tst:=tst+st[t];
    inc(t);
  end;
  inc(t);
  delimString:=tst;
end;

Procedure convert(fontName:string);
var
	fText:text;
	f:file;
	pcxHeader:tPcxHeader;
	pcxPalette:tPcxPal256;
	dataLine:string;
	fBuffer,fPoint,fBufferEnd,pBufferEnd,
	pBuffer,pPoint:^byte;
	d,countWhite,countBlack:byte;
	n:word;
	command,
	fBufferSize,pBufferSize:dword;
	info:tglkFontheader;
	convertCount:dword;
	
begin
	assign(f,fontName+'.pcx');
		reset(f,1);

		blockread(f,pcxHeader,sizeof(tPcxHeader));
		fBufferSize:=filesize(f)-896;
		getmem(fBuffer,fBufferSize);
		seek(f,128);
		blockread(f,fBuffer^,fBufferSize);
		seek(f,filesize(f)-768);
		blockread(f,pcxPalette,768);
	close(f);
	with info do begin
		{
			hRes and vRes are NOT the actual width and height
			so calculate off min/max values!
		}
		bufferWidth:=(PcxHeader.xMax-PcxHeader.xMin)+1;
		bufferHeight:=(PcxHeader.yMax-PcxHeader.yMin)+1;
		pBufferSize:=bufferWidth*bufferHeight;
		charWidth:=bufferWidth div 32;
		charHeight:=bufferHeight div 3;
		writeln('Parsing    : ',fontname,'.pcx ',bufferWidth,'x',bufferHeight);
		writeln('Glyph Size : ',charWidth,'x',charHeight);
	end;
	
	getmem(pBuffer,pbufferSize);
	
	fPoint:=fBuffer;
	fBufferEnd:=fPoint+fBufferSize;
	pPoint:=pBuffer;
	pBufferEnd:=pPoint+pBufferSize;
	
	writeln('Beginning PCX RLE Decompress');
	
	while not(fpoint=fBufferEnd) do begin
		if (fPoint^ and 192)=192 then begin
			d:=fPoint^ and 63;
			inc(fPoint);
			while (d>0) do begin
				pPoint^:=pcxPalette[fPoint^].red;
				dec(d);
				inc(pPoint);
			end;
		end else begin
			pPoint^:=pcxPalette[fPoint^].red;
			inc(pPoint);
		end;
		inc(fPoint);
	end;
	
	writeln('PCX RLE Decompress Complete');
	
	freeMem(fBuffer);
	
	writeln('texture loaded ok, loading ',fontName,'.kerning');
	assign(fText,fontName+'.kerning');
	reset(fText);
	while not(eof(fText)) do begin
		readln(fText,dataLine);
		if (length(dataLine)>0) and not(dataline[1]=';') then begin
			n:=1;
			val(delimString(dataLine,',',n),command);
			case trunc(command) of 
				32..127:begin
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][0]
					);
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][1]
					);
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][2]
					);	
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][3]
					);
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][4]
					);
					val(
						delimString(dataLine,',',n),
						info.kerningTable[command][5]
					);
				end;
			end;
		end;
	end;
	close(fText);
	
	writeln('Kerning processing complete');
	
	assign(f,fontName+'.glkfont');
	rewrite(f,1);
	
	writeln('Writing glkfont header');
	blockwrite(f,info,sizeof(info));
	
	countBlack:=0;
	countWhite:=0;
	pPoint:=pBuffer;
	
	writeln('Encoding/writing glkfont - ',pBufferSize,' pixels');
	
	convertCount:=sizeof(info);
	
	while not(pPoint=pBufferEnd) do begin
		case (pPoint^) of 
			black:begin
				if countWhite>0 then begin
					convertCount+=2;
					blockwrite(f,white,1);
					blockwrite(f,countWhite,1);
					countWhite:=0;
				end;
				inc(countBlack);
				if (countBlack=$FF) then begin
					convertCount+=2;
					blockwrite(f,black,1);
					blockwrite(f,countBlack,1);
					countBlack:=0;
				end;
			end;
			white:begin
				if countBlack>0 then begin
					convertCount+=2;
					blockwrite(f,black,1);
					blockwrite(f,countBlack,1);
					countBlack:=0;
				end;
				inc(countWhite);
				if countWhite=$FF then begin
					convertCount+=2;
					blockwrite(f,white,1);
					blockwrite(f,countWhite,1);
					countWhite:=0;
				end;
			end;
			else begin
				if countBlack>0 then begin
					convertCount+=2;
					blockwrite(f,black,1);
					blockwrite(f,countBlack,1);
					countBlack:=0;
				end;
				if countWhite>0 then begin
					convertCount+=2;
					blockwrite(f,white,1);
					blockwrite(f,countWhite,1);
					countWhite:=0;
				end;
				convertCount+=1;
				blockwrite(f,pPoint^,1);
			end;
		end;
		inc(pPoint);
	end;
	
	if countBlack>0 then begin
		convertCount+=2;
		blockwrite(f,0,1);
		blockwrite(f,countBlack,2);
		countBlack:=0;
	end;
	
	if countWhite>0 then begin
		convertCount+=2;
		blockwrite(f,255,1);
		blockwrite(f,countWhite,1);
		countWhite:=0;
	end;
	
	close(f);
	
	writeln('Conversion Complete');
	
	freeMem(pBuffer);
end;

function fileExists(name:string):boolean;
var
   f:Text;
begin
   Assign(f,name);
{$I-}
   Reset(f);
{$I+}
	if IOResult=0 then begin
		close(f);
		fileExists:=true;
	end else fileExists:=false;
end;

begin
	if (paramCount=1) then begin
		if (
			fileExists(paramStr(1)+'.pcx') and
			fileExists(paramStr(1)+'.kerning')
		) then begin
			convert(paramStr(1));
		end else begin
			writeln('ERROR - Either the .pcx or .kerning file are missing');
		end;
	end else begin
		if (paramCount>1) then begin
			writeln('ERROR - Too many Parameters'); 
			writeln;
		end;
		writeln('FontConvert.exe - convert file.pcx and file.kerning into file.glkfont');
		writeln('Calling paramters');
		writeln('  fontconvert filePrefix');
		writeln;
		writeln('For example if you have a "test.pcx" and "test.kerning" and want to create a');
		writeln('"test.glkfont", you would enter "fontconvert test"');
	end;
end.