{$COPERATORS ON}
{$MODE DELPHI} 

program game;

uses
	{$ifdef Windows}
		windows,
	{$endif}
	sdl,
	gl,
	glKernedFont;
	
const
	cgaColors:array[0..15,0..2] of glFloat=(
		(  0,  0,  0), // Black
		(  0,  0,2/3), // Blue
		(  0,2/3,  0), // Green
		(  0,2/3,2/3), // Cyan
		(2/3,  0,  0), // Red
		(2/3,  0,2/3), // Magenta
		(2/3,1/3,  0), // Brown
		(2/3,2/3,2/3), // Light Gray
		(1/3,1/3,1/3), // Dark Gray
		(1/3,1/3,1.0), // Light Blue
		(1/3,1.0,1/3), // Light Green
		(1/3,1.0,1.0), // Light Cyan
		(1.0,1/3,1/3), // Light Red
		(1.0,1/3,1.0), // Light Magenta
		(1.0,1.0,1/3), // Yellow
		(1.0,1.0,1.0)  // White
	);

var
	defaultWidth,defaultHeight:longint;
	minX,minY,maxX,maxY:glFloat;
	screen:pSDL_SURFACE;
	SDLTestEvent:pSDL_EVENT;
	timer360offset:glFloat;
	demoFont:tglKernedFont;

	
function timer360:glFloat;
var
	t,n:glFloat;
begin
	t:=SDL_getTicks/(6000/360);
	n:=t-timer360Offset;
	while (n>360) do n-=360;
	timer360Offset:=n-t;
	timer360:=n;
end;

procedure glCgaColor(color:byte);
begin
	glColor3F(
		cgaColors[color and $0F,0],
		cgaColors[color and $0F,1],
		cgaColors[color and $0F,2]
	);
end;
	
procedure cgaStripe(st:string);
var
	t,n:word;
begin
	t:=0;
	n:=0;
	while (t<length(st)) do begin
		inc(t);
		if not(st[t]=' ') then begin
			inc(n);
		end;
		glCgaColor(n);
		if n=15 then n:=0;
		demoFont.writeChar(st[t]);
	end;
end;

function strDouble(t:double; n,p:longint):string;
var
  st:string;
begin
  str(t:n:p,st);
  strDouble:=st;
end;

procedure varsInit();
var
	ratio:glFloat;
begin

	minY:=0;
	maxY:=25;
	minX:=0;
	ratio:=defaultWidth/defaultHeight;
	maxX:=25*ratio;

	glViewport(0,0,defaultWidth,defaultHeight);
	glmatrixmode(GL_PROJECTION);
	glOrtho(minX,maxX,maxY,minY,0,$FFFF);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glDepthRange(0,$FFFF);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glShadeModel(GL_SMOOTH);
	glClearColor(0,0,0,0);
	glClearDepth(65535);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	demoFont.init('test_36x60',1);
	new(SDLTestEvent);
end;

procedure varsTerm();
begin
	demoFont.term;
	dispose(SDLTestEvent);
end;

function SDL_poll:boolean;
var
	windowEnd:boolean;
begin
	windowEnd:=false;
	if SDL_pollEvent(SDLTestEvent)=1 then begin
		case SDLTestEvent^.type_ of
			SDL_KEYDOWN:begin
				if (SDLTestEvent^.key.keysym.sym=SDLK_ESCAPE) then begin
					windowEnd:=true;
				end;
			end;
			SDL_QUITEV:begin
				windowEnd:=true;
			end;
		end;
	end;
	SDL_poll:=windowEnd;
end;

procedure drawTest();
var
	t:word;
	tst:string;
	menuRotate,textYPos,rotR,rotG,rotB,rotCalc:glFloat;
begin

	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
	
	textYPos:=0.5;
	
	glColor3f(1,1,1);
	demoFont.scale(1);
	
	demoFont.setPosition(0.5,textYPos,0);
	demoFont.writeString('Kerned OpenGL Texture Fonts Demo');
	
	demoFont.setPosition(maxX-0.5,textYPos,0);
	demoFont.writeStringRight('Press <ESC> to Exit');
	
	textYPos+=2;
	
	demoFont.setPosition(0.5,textYPos,0);
	demoFont.writeString('Kerning Test: ToT TLT TVB LV YoY YrY TrT TgT Lj TJ Tj jjj');
	
	textYPos+=2;
	
	for t:=2 to 16 do begin
		glcolor3f((t-2)/14,1-((t-2)/14),0);
		demoFont.setPosition(0.5,textYPos,0);
		demoFont.scale(0.125*t);
		demoFont.writeString(strDouble(12.5*t,0,1)+'% Text Size');
		textYPos+=t*0.125;
	end;
	
	textYPos+=1.5;
	
	demoFont.scale(1);
	tst:='Colors are easily added using glColor3f, should work with multitexturing!';
	demoFont.setPosition((maxX-demoFont.stringSize(tst))/2,textYPos,0);
	cgaStripe(tst);
	
	
	menuRotate:=timer360;
	if menuRotate<120 then begin // blue to red
		rotR:=menuRotate/120;
		rotG:=0;
		rotB:=1-rotR;
	end else if (menuRotate<240) then begin // red to green
		rotCalc:=menuRotate-120;
		rotG:=rotcalc/120;
		rotR:=1-rotG;
		rotB:=0;
	end else begin // green to blue
		rotCalc:=menuRotate-240;
		rotB:=rotcalc/120;
		rotG:=1-rotB;
		rotR:=0;
	end;
	
	demoFont.scale(2);
	glColor3F(rotR,rotG,rotB);
	glPushMatrix;
		glTranslatef(maxX-10.5,12.5,0);
		glRotatef(menuRotate,0,0,1);
		demoFont.setPosition(0,0,0);
		demoFont.writeStringCentered('Text can rotate off center');
	glPopMatrix;

	demoFont.scale(1.5);
	glColor3F(rotG,rotB,rotR);
	glPushMatrix;
		glTranslatef(maxX-12,15,0);
		glRotatef(menuRotate*2+120,0,0,1);
		demoFont.setPosition(0,0,0);
		demoFont.writeString('Text can rotate off top left edge');
	glPopMatrix;
	
	demoFont.scale(1);
	glColor3F(rotB,rotR,rotG);
	glPushMatrix;
		glTranslatef(maxX-15,13,0);
		glRotatef(menuRotate*3+240,0,0,1);
		demoFont.setPosition(0,0,0);
		demoFont.writeStringRight('Text can rotate off top right edge');
	glPopMatrix;
	
	glFlush();

	SDL_GL_SWAPBUFFERS;

end;

begin

	{$ifdef Windows}
		defaultWidth:=GetSystemMetrics(SM_CXSCREEN)*3 div 4;
		defaultHeight:=GetSystemMetrics(SM_CYSCREEN)*3 div 4;
	{$else}
		defaultWidth:=800;
		defaultHeight:=600;
	{$endif}	
	timer360offset:=0;
	
	SDL_INIT(SDL_INIT_VIDEO);
	
	sdl_gl_setAttribute(SDL_GL_RED_SIZE,8);
	sdl_gl_setAttribute(SDL_GL_GREEN_SIZE,8);
	sdl_gl_setAttribute(SDL_GL_BLUE_SIZE,8);
	sdl_gl_setAttribute(SDL_GL_ALPHA_SIZE,8);
	sdl_gl_setAttribute(SDL_GL_DEPTH_SIZE,16);
	sdl_gl_setAttribute(SDL_GL_DOUBLEBUFFER,1);
	sdl_gl_setAttribute(SDL_GL_STENCIL_SIZE,8);
	
	screen:=SDL_SETVIDEOMODE(defaultWidth,defaultHeight,32,SDL_OPENGL);

	if screen=nil then begin
		writeln('Failed to initialize SDL video');
	end else begin
	
		varsInit;
		repeat 
			drawTest();
		until SDL_poll;
		varsTerm;
		
		SDL_FREESURFACE(screen);
		SDL_QUIT;
		
	end;
end.
